!**********************************************************************!
      SUBROUTINE DROP_COOLING_RATE(TEMP_RATE,DIAMETER,THE_TEMP,
     & GAS_TEMP,WALL_TEMP,DROP_SPEC_HEAT,DENSITY,
     & EMISSIVITY,STEF_BOLT,HEAT_TRANS_COEFF,TOT_COMP,
     & RADI_RATE,CONVEC_RATE)
      IMPLICIT NONE
! calculate the cooling rate of a liquid droplet of a given temperature
! accounting for both radiation and convection.
!----------------------------------------------------------------------!
! declare variables
      REAL DIAMETER,GAS_TEMP,WALL_TEMP,DROP_SPEC_HEAT
      REAL DENSITY,TEMP_RATE,EMISSIVITY,CONVEC_RATE
      REAL RADI_RATE,HEAT_TRANS_COEFF
      DOUBLE PRECISION THE_TEMP,TOT_COMP,STEF_BOLT
      DIMENSION TOT_COMP(12)
!----------------------------------------------------------------------!
! calculate the rate of heat loss by convection
      CONVEC_RATE = -6.0E+00*HEAT_TRANS_COEFF*(THE_TEMP - GAS_TEMP)
     & / (DENSITY * DIAMETER)
!----------------------------------------------------------------------!
! calculate the rate of heat loss by radiation
      RADI_RATE = -6.0E+00 * EMISSIVITY * STEF_BOLT *
     & ( (THE_TEMP+2.7315E+02)**4.0E+00 -
     & (WALL_TEMP+2.7315E+02)**4.0E+00 ) / (DENSITY * DIAMETER)
!----------------------------------------------------------------------!
! calculate the rate of temperature change
      TEMP_RATE = (CONVEC_RATE + RADI_RATE) / DROP_SPEC_HEAT
!----------------------------------------------------------------------!
! 201  FORMAT(SP,3(E10.2,1X),E10.2)
!----------------------------------------------------------------------!
      END SUBROUTINE DROP_COOLING_RATE
!**********************************************************************!
! subroutine to start MTData.  The values are specifi to MTData and have
! no meaning apart from to MTData, as described i nthe MTData
! Application Programming Interface User Manual
      SUBROUTINE START_MTDATA(IMODE,MPIFILE,MPRFILE)
      IMPLICIT NONE
      INCLUDE 'DIRUSRAP.FOR'
!**********************************************************************!
! declare variables.
      INTEGER IMODE,NERR
!----------------------------------------------------------------------!
      CHARACTER*20 MPIFILE,MPRFILE
!----------------------------------------------------------------------!
! initialise the error indicator NERR to zero.
      NERR = 0
!----------------------------------------------------------------------!
      CALL INITIALISE_MTDATA(0)
      CALL SGUMEN(1)
      CALL SGUMEN(7)
      CALL MTOPTN(IMODE,'STAGE_1=NEW')
      CALL OPEN_MPI_FILE(MPIFILE,MPRFILE,NERR)
      IF(NERR .GT. 0 .OR. NERR .LT. 0) THEN
        PRINT *, 'Error opening MPI or MPR file.  Program exits safely.'
        STOP
      END IF
      CALL SET_PRINT_LEVEL(MINIMAL)
!----------------------------------------------------------------------!
      END SUBROUTINE START_MTDATA
! calculate the Gibbs energy of a single phase at a given composition.
!**********************************************************************!
      SUBROUTINE DRIVE(THIS_TEMP,THIS_COMP,THIS_PHASE,NO_PHASES,
     & BLANKPHASE)
      IMPLICIT NONE
      INCLUDE 'DIRUSRAP.FOR'
!----------------------------------------------------------------------!
      INTEGER NO_PHASES,I2,ERRNO
      DOUBLE PRECISION THIS_TEMP,THIS_COMP
      DIMENSION THIS_COMP(12)
      CHARACTER*20 THIS_PHASE,BLANKPHASE
!----------------------------------------------------------------------!
      ERRNO      = 0
! set the composition.
      CALL SETCOMP(THIS_COMP)
! SET THIS_PHASE TO BE IN THE CALCULATION AND EXCLUDE ALL OTHER PHASES.
      CALL SETPHASES(THIS_PHASE,BLANKPHASE,BLANKPHASE,NO_PHASES)
! SET TEMPERATURE.
      CALL SET_TEMPERATURE(THIS_TEMP+2.7315E+02)
! CALCULATE THE EQUILIBIRUM.
      CALL COMPUTE_EQUILIBRIUM()
! IF ERRORS HAVE OCCURRED, PRINT OUT THE MTDATA ERROR, A MESSAGE AND
! STOP THE PROGRAM.
      IF(TEST_ERROR(ERRNO) .EQV. .TRUE.) THEN
        CALL LOG_ERROR(6)
        PRINT *, 'MTData has encountered an error.  Program exits ',
     & 'safely.'
        STOP
      END IF
!----------------------------------------------------------------------!
      END SUBROUTINE DRIVE
!**********************************************************************!
      SUBROUTINE SETCOMP(THE_COMP)
      IMPLICIT NONE
      INCLUDE 'DIRUSRAP.FOR'
!----------------------------------------------------------------------!
      INTEGER I4
      DOUBLE PRECISION THE_COMP,BALANCE
      DIMENSION THE_COMP(12)
!----------------------------------------------------------------------!
! set the initial composition, pressure and temperature in MTData.
! define a variable that is the sum of the masses of all solute
! additions.
      BALANCE=1.0E+02
      COMPSET:DO I4=2,INIT_NO_OF_COMPONENTS(),1
! for each solute, define the mass in the system and0.1THE_COMP(I4))
        CALL SET_INIT_COMPONENT_MASS(I4,THE_COMP(I4))
        BALANCE = BALANCE - THE_COMP(I4)
      END DO COMPSET
! set the mass of iron to the balance.
      CALL SET_INIT_COMPONENT_MASS(1,BALANCE)
      END SUBROUTINE SETCOMP
!**********************************************************************!
      SUBROUTINE SETPHASES(FIRSTPHASE,SECONDPHASE,THIRDPHASE,NO_PHASES)
      IMPLICIT NONE
      INCLUDE 'DIRUSRAP.FOR'
!----------------------------------------------------------------------!
      INTEGER NO_PHASES,I3
      CHARACTER*20 FIRSTPHASE,SECONDPHASE,THIRDPHASE
!----------------------------------------------------------------------!
! GO THROUGH EACH POSSIBLE PHASE AND EXCLUDE ALL EXCEPT THE ONE(S) OF
! INTEREST
      PHASECHECKER:DO I3=1,NO_PHASES,1
        IF(INIT_PHASE_NAME(I3)==FIRSTPHASE .OR. 
     & INIT_PHASE_NAME(I3)==SECONDPHASE .OR.
     & INIT_PHASE_NAME(I3)==THIRDPHASE) THEN
          CALL SET_INIT_PHASE_CLASS(I3,NORMAL)
        ELSE
          CALL SET_INIT_PHASE_CLASS(I3,ABSENT)
        END IF
      END DO PHASECHECKER
!----------------------------------------------------------------------!
      END SUBROUTINE SETPHASES
!**********************************************************************!
      SUBROUTINE FIND_LIQ(BULK_COMP,LIQFER,LIQAUS,LIQUID,AUSTENITE,
     & FERRITE,HOWMANY)
      IMPLICIT NONE
      INCLUDE 'DIRUSRAP.FOR'
!----------------------------------------------------------------------!
      INTEGER I6,HOWMANY,NO_FER,NO_AUS,NO_LIQ,ERRNO,j6
      DOUBLE PRECISION SOME_TEMP,LIQFER,LIQAUS,BULK_COMP,TOLERANCE
! SET THE MAXIMUM TOLERABLE PHASE FRACTION OF SOLID THAT REPRESENTS THE
! LIQUIDUS.  1 mol.% SOLID GIVES THE LIQUIDUS TO WITHIN APPROXIMATELY
! 1 DEGREE.
      PARAMETER(TOLERANCE=1.0D-02)
      DOUBLE PRECISION SOLIDFRAC
      DIMENSION BULK_COMP(12),SOME_TEMP(3),SOLIDFRAC(3)
      CHARACTER*20 LIQUID,AUSTENITE,FERRITE,BLANKPHASE
!----------------------------------------------------------------------!
! SET THE UPPER BOUND OF TEMPERATURE TO 2000 C. 
      SOME_TEMP(1) = 2.27315D+03
! SET THE LOWER BOUND OF TEMPRATURE TO 0 C.
      SOME_TEMP(3) = 1.27325D+03
      BLANKPHASE = 'abcdefghijklmnopqrst'

      CALL SETCOMP(BULK_COMP)
!----------------------------------------------------------------------!
! find the liquidus assuming ferrite forms from the melt
      CALL SETPHASES(LIQUID,FERRITE,BLANKPHASE,HOWMANY)

      DO I6=1,3,1
        SOLIDFRAC(I6) = 0.0D+00
      END DO
! test all possible phases.  If the phase name mathces the phase you
! want, grab the number that identifies that phase.
      DO I6=1,HOWMANY,1 
        IF(INIT_PHASE_NAME(I6) == FERRITE) THEN
          NO_FER = I6
        END IF
        IF(INIT_PHASE_NAME(I6) == LIQUID) THEN 
          NO_LIQ = I6
        END IF
      END DO

      LIQ_FER:DO
! set the middle temperature to be the mean of the current bounds.
        SOME_TEMP(2) = 0.5D+00 * (SOME_TEMP(1) + SOME_TEMP(3)) 

        DO I6=1,3,1
! set the temperature to each of the upper bound, middle and lower bound
! in turn.
          CALL SET_TEMPERATURE(SOME_TEMP(I6))
! compute the equilibirum for the selected temperature.
          CALL COMPUTE_EQUILIBRIUM()
! IF ERRORS HAVE OCCURRED, PRINT OUT THE MTDATA ERROR, A MESSAGE AND
! STOP THE PROGRAM.
          IF(TEST_ERROR(ERRNO) .EQV. .TRUE.) THEN
            CALL LOG_ERROR(6)
            PRINT *, 'MTData has encountered an error.  Program exits ',
     & 'safely.'
            STOP
          END IF
! calculate the molar fraction of the solid phase.
          SOLIDFRAC(I6) = MOLES_OF_COMPONENTS_IN_PHASE(NO_FER,1) / 
     & SUM_OF_COMPONENT_AMOUNT()
        END DO
! test if the amount of solid is greater than zero and less than
! TOLERANCE - this means the temperature is very slightly below the
! liquidus, but close enough.
        IF(SOLIDFRAC(2).GT.0.0D+00 .AND. SOLIDFRAC(2).LE.TOLERANCE) THEN
! accept the temperature as the liquidus for formation of ferrite.  
! Convert to Celsius.
          LIQFER = SOME_TEMP(2)-2.7315D+02
! stop trying to refine the temperature.
          EXIT LIQ_FER
! test if too much solid has formed - in other words, the temperature is
!too low.
        ELSE IF(SOLIDFRAC(2) .GT. TOLERANCE) THEN
! and if so, et the lower bound to the old mean value.
          SOME_TEMP(3) = SOME_TEMP(2)
        ELSE
! if no solid has formed, the temerature is too high, so set the upper
! bound to the old mean value.
          SOME_TEMP(1) = SOME_TEMP(2)
        END IF
      END DO LIQ_FER
!----------------------------------------------------------------------!
! find the liquidus assuming austenite forms directly from the melt.
! SET THE UPPER BOUND OF TEMPERATURE TO 2000 C. 
      SOME_TEMP(1) = 2.27315D+03
! SET THE LOWER BOUND OF TEMPRATURE TO 0 C.
      SOME_TEMP(3) = 1.27325D+03

      CALL SETPHASES(LIQUID,AUSTENITE,BLANKPHASE,HOWMANY)

! test all possible phases.  If the phase name mathces the phase you
! want, grab the number that identifies that phase.
      DO I6=1,HOWMANY,1
        IF(INIT_PHASE_NAME(I6) == AUSTENITE) THEN
          NO_AUS = I6
        END IF
      END DO

      LIQ_AUS:DO
! set the middle temperature to be the mean of the current bounds.
        SOME_TEMP(2) = 0.5D+00 * (SOME_TEMP(1) + SOME_TEMP(3))
        DO I6=1,3,1
! set the temperature to each of the upper bound, middle and lower bound
! in turn.
          CALL SET_TEMPERATURE(SOME_TEMP(I6))
! compute the equilibirum for the selected temperature.
          CALL COMPUTE_EQUILIBRIUM()
! calculate the molar fraction of the solid phase.
          SOLIDFRAC(I6) = MOLES_OF_COMPONENTS_IN_PHASE(NO_AUS,INITIAL) / 
     & SUM_OF_COMPONENT_AMOUNT()
        END DO
! test if the amount of solid is greater than zero and less than
! TOLERANCE - this means the temperature is very slightly below the
! liquidus, but close enough.
        IF(SOLIDFRAC(2).GT.0.0D+00 .AND. SOLIDFRAC(2).LE.TOLERANCE) THEN
! accept the temperature as the liquidus for formation of ferrite.
          LIQAUS = SOME_TEMP(2)-2.73125D+02
! stop trying to refine the temperature.
          EXIT LIQ_AUS
! test if too much solid has formed - in other words, the temperature is
! too low.
        ELSE IF(SOLIDFRAC(2) .GT. TOLERANCE) THEN
! set the lower bound to the old mean value.
          SOME_TEMP(3) = SOME_TEMP(2)
        ELSE
! if no solid has formed, the temerature is too high, so set the upper
! bound to the old mean value.
          SOME_TEMP(1) = SOME_TEMP(2)
        END IF
      END DO LIQ_AUS
!----------------------------------------------------------------------!
      END SUBROUTINE FIND_LIQ
!**********************************************************************!
      SUBROUTINE NUCIT(NUCTEMP,DTDT,LIQTEMP,DROPSIZE,PI,AVOGADRO,
     & BOLTZMANN,SYSLATENT,SYSVOL,NUMBER_NUCLEI)
      IMPLICIT NONE
      INCLUDE 'DIRUSRAP.FOR'
!----------------------------------------------------------------------!
      INTEGER I7
      REAL DTDT,DROPSIZE
! parameters that are calculated previously and passed to this
! subroutine for use in the calculations.
      DOUBLE PRECISION NUCTEMP,LIQTEMP,SYSLATENT
! volumes that are calculated in this subroutine.
      DOUBLE PRECISION SYSVOL,MOLVOL,ATOMVOL,DROPVOL
! latent heats per mole and atom, respectively - calculated from LATENT
! in this subroutine.
      DOUBLE PRECISION MOLLATENT,ATOMLATENT
! other values calculated as part of this subroutine
      DOUBLE PRECISION UNDERCOOL,INTERFACIAL,NUCRATE,NUMBER_NUCLEI
      DOUBLE PRECISION PI,AVOGADRO,BOLTZMANN
!----------------------------------------------------------------------!
      MOLLATENT = SYSLATENT / SUM_OF_COMPONENT_AMOUNT()
      ATOMLATENT = MOLLATENT / AVOGADRO
!----------------------------------------------------------------------!
      MOLVOL = SYSVOL / SUM_OF_COMPONENT_AMOUNT()
      ATOMVOL = MOLVOL / AVOGADRO
!----------------------------------------------------------------------!
      DROPVOL = PI * DROPSIZE**3.0D+00 / 6.0D+00
!----------------------------------------------------------------------!
      UNDERCOOL = LIQTEMP - NUCTEMP
!----------------------------------------------------------------------!
      INTERFACIAL = -4.5D-01 * ATOMLATENT * ATOMVOL**(-2.0D+00/3.0D+00)
 !----------------------------------------------------------------------!
      NUCRATE = 1.0D+41 * EXP(-(1.6D+01 * PI * INTERFACIAL**2.0D+00 * 
     & MOLVOL**2.0D+00 * (LIQTEMP+2.7315D+02)**2) / (3.0D+00 * 
     & BOLTZMANN * (NUCTEMP+2.7315D+02) * MOLLATENT**2.0D+00 * 
     & UNDERCOOL**2.0D+00))
!----------------------------------------------------------------------!
      NUMBER_NUCLEI = 1.0D-02 * NUCRATE * SYSVOL * UNDERCOOL / 
     & (-1.0D+00*DTDT)
!----------------------------------------------------------------------!
      END SUBROUTINE NUCIT
!**********************************************************************!
      SUBROUTINE RECALESCENCE(EVOLVE_RATE,NOWTEMP,SOLIDFRAC,PPTSIZE,
     & THE_DIAMETER,T_LIQ,LIQ_COMP,LIQUID,SOLID,DROP_DENSE,PI,
     & RECAL_T_STEP)
      IMPLICIT NONE
      INCLUDE 'DIRUSRAP.FOR'
!----------------------------------------------------------------------!
      INTEGER I8
! variables passed to/from the main program
      DOUBLE PRECISION EVOLVE_RATE,NOWTEMP,LIQ_COMP,DROP_DENSE,PI
! variables passed to/from the main program
      DOUBLE PRECISION THE_DIAMETER,PPTSIZE,T_LIQ,SOLIDFRAC
! local variables
      DOUBLE PRECISION DFDX,DXDT,DFDT,LIQUID_G,SOLID_G
      DOUBLE PRECISION RECAL_T_STEP
      DIMENSION LIQ_COMP(11)
      CHARACTER*20 LIQUID,SOLID,BLANKPHASE
!----------------------------------------------------------------------!
! initialise local variables
      DFDX = 0.0D+00
      DXDT = 0.0D+00
      LIQUID_G = 0.0D+00
      SOLID_G = 0.0D+00
      BLANKPHASE = 'abcdefghijklmnopqrst'
!----------------------------------------------------------------------!
! calculate the volume fraction of the particle consumed by a
! precipitate of a given size (geometrical constructuion)
      DFDX = 1.5D+00 * (PPTSIZE * THE_DIAMETER**(-2.0D+00)) * ( 2.0D+00 
     & + (PPTSIZE/THE_DIAMETER))
! calculate the growth velocity, assessed to be 0.01 * undercooling
      DXDT = 1.0D-02 * (T_LIQ - NOWTEMP)
! find the rate of change of volume fraction of the solid, eual to the
! product of growth velocity and vol. frac. of droplet of given size
! (chain rule)
      DFDT = DFDX * DXDT
! find the heat of fusion - simplify be neglecting density differences
! between liquid and solid
      CALL SET_TEMPERATURE(NOWTEMP+2.7315D+02)
! calculate the specific enthalpy of liquid in the droplet
      CALL SETPHASES(LIQUID,BLANKPHASE,BLANKPHASE,INIT_NO_OF_PHASES())
      CALL COMPUTE_EQUILIBRIUM()
! SPECIFIC GIBBS ENERGY OF LIQUID
      LIQUID_G = SYSTEM_GIBBS_ENERGY() / SUM_OF_COMPONENT_MASS()
! calculate the specific enthalpy of solid in the droplet
      CALL SETPHASES(SOLID,BLANKPHASE,BLANKPHASE,INIT_NO_OF_PHASES())
      CALL COMPUTE_EQUILIBRIUM()
! SPECIFIC GIBBS ENERGY OF SOLID
      SOLID_G = SYSTEM_GIBBS_ENERGY() / SUM_OF_COMPONENT_MASS()
! multiply the density by the volume to give the totoal mass, then by
! the rate of change of solid fraction to get the rate of change of
! solid fraction and then multiply by the sepcific enthalpy change to
! get the absolute enthalpy change. Neglect density chagnes to simplify
! the maths.
!
! PREDICTED SPECIFIC HEAT EVOLUTION.  THE FACTOR OF -1 ACCOUTNS FOR THE
! FACT THAT WE ARE INTERESTED IN THE ENERGY THE SYSTEM HAS lost, NOT THE
! CHANGE IN ENERGY OF THE SYSTEM ITSELF.
      EVOLVE_RATE = DFDT * DROP_DENSE * (-1.0D+00)*(SOLID_G - LIQUID_G)
      PPTSIZE = PPTSIZE + DXDT * RECAL_T_STEP
      SOLIDFRAC = SOLIDFRAC + DFDT * RECAL_T_STEP
!----------------------------------------------------------------------!
      END SUBROUTINE RECALESCENCE
!**********************************************************************!
!      SUBROUTINE PARTITION(THE_COMP,LIQUID,SOLIDPHASE,BLANKPHASE,
!     & NOWTEMP,K,KAVERAGE)
!      IMPLICIT NONE
!      INCLUDE 'DIRUSRAP.FOR'
!      INTEGER I9,LIQNO,SOLNO
!      DOUBLE PRECISION THE_COMP,NOWTEMP,K,KAVERAGE,KCONTRIB
!      DIMENSION THE_COMP(12),K(11),KCONTRIB(11)
!      CHARACTER*20 LIQUID,SOLIDPHASE,BLANKPHASE

!      LIQNO    = 0
!      SOLNO    = 0
!      KAVERAGE = 0.D+00
!      DO I9=1,INIT_NO_OF_COMPONENTS()-1,1
!        K(I9)        = 0.0D+00
!        KCONTRIB(I9) = 0.0D+00
!      END DO

!      CALL SET_TEMPERATURE(NOWTEMP+2.7315D+02)
!      CALL SETPHASES(LIQUID,SOLIDPHASE,BLANKPHASE,2)
!      CALL SETCOMP(THE_COMP)

!      CALL COMPUTE_EQUILIBRIUM()

!      LIQNO = INIT_PHASE_NO(LIQUID)
!      SOLNO = INIT_PHASE_NO(SOLIDPHASE)
!      PRINT *, 'Liquid ID number = ',LIQNO,'. Solid ID number = ',SOLNO
!      KFINDER:DO I9=2,INIT_NO_OF_COMPONENTS(),1
!        K(I9-1) = INIT_COMPONENT_NO(),' ',
!     & COMPONENT_W_OF_PHASE(I9,LIQNO,INITIAL) / 
!     & COMPONENT_W_OF_PHASE(I9,SOLNO,INITIAL)
!        PRINT *, COMPONENT_W_OF_PHASE(I9,LIQNO,INITIAL),' ',
!     & COMPONENT_W_OF_PHASE(I9,SOLNO,INITIAL),' ',K(I9-1)
!      END DO KFINDER
!      AVERAGER:DO I9=2,INIT_NO_OF_COMPONENTS(),1
!        KCONTRIB(I9-1) = K(I9-1) * INIT_COMPONENT_MASS(I9) / 
!     & (SUM_OF_COMPONENT_MASS() - INIT_COMPONENT_MASS(1))
!        KAVERAGE = KAVERAGE + KCONTRIB(I9-1)
!        PRINT *, K(I9-1),' ',INIT_COMPONENT_MASS(I9),' ',
!     & (SUM_OF_COMPONENT_MASS() - INIT_COMPONENT_MASS(1)),' ',KAVERAGE
!      END DO AVERAGER

!      END SUBROUTINE PARTITION
!**********************************************************************!
      SUBROUTINE SCHEIL(LIQ_COMP,STARTTEMP,ENDTEMP,LIQUID,SOLIDPHASE,
     & BLANKPHASE,SOLID_COMP,DELTA_G,DFDT,TIMESTEP,LIQNO,SOLIDFRAC,K10)
      IMPLICIT NONE
      INCLUDE 'DIRUSRAP.FOR'
!----------------------------------------------------------------------!
      INTEGER I10,SOLNO,LIQNO,K10
      DOUBLE PRECISION LIQ_COMP,STARTTEMP,ENDTEMP,GIBBS,DELTA_G,DFDT
      DOUBLE PRECISION NEW_SOLID_MASS,SOLID_COMP,TIMESTEP,SOLIDFRAC
      DIMENSION LIQ_COMP(12),SOLID_COMP(12,4),GIBBS(2)
      CHARACTER*20 LIQUID,SOLIDPHASE,BLANKPHASE
!----------------------------------------------------------------------!
      NEW_SOLID_MASS = 0.0D+00
      SOLNO = INIT_PHASE_NO(SOLIDPHASE)
      LIQNO = INIT_PHASE_NO(LIQUID)
! CALCULATE THE FREE ENERGY OF LIQUID AT THE CURRENT STEP'S STARTING
! TEMPERATURE AND THE LIQUID COMPOSITION AT THE START OF THE STEP
      DO I10=1,INIT_NO_OF_PHASES(),1
        IF(INIT_PHASE_NAME(I10) == LIQUID) THEN
        CALL SET_INIT_PHASE_CLASS(I10,NORMAL)
        ELSE
          CALL SET_INIT_PHASE_CLASS(I10,ABSENT)
        END IF
      END DO
      CALL SETCOMP(LIQ_COMP)
      DO I10=1,INIT_NO_OF_COMPONENTS(),1
        CALL SET_INIT_COMPONENT_MASS(I10,LIQ_COMP(I10))
      END DO
      CALL SET_TEMPERATURE(STARTTEMP)
      CALL COMPUTE_EQUILIBRIUM()
      GIBBS(1) = SYSTEM_GIBBS_ENERGY() / SUM_OF_COMPONENT_MASS()
      DO I10=1,INIT_NO_OF_PHASES(),1
        IF(INIT_PHASE_NAME(I10) == SOLIDPHASE . OR. 
     & INIT_PHASE_NAME(I10) == LIQUID) THEN
          CALL SET_INIT_PHASE_CLASS(I10,NORMAL)
        ELSE
          CALL SET_INIT_PHASE_CLASS(I10,ABSENT)
        END IF
      END DO
      CALL SET_TEMPERATURE(ENDTEMP)
      CALL COMPUTE_EQUILIBRIUM()
      GIBBS(2) = SYSTEM_GIBBS_ENERGY() / SUM_OF_COMPONENT_MASS()
      DELTA_G = GIBBS(2) - GIBBS(1)
! the 'system' seen by this subroutine includes ONLY the remaining
! liquid and gradually shrinks as the subroutine is repeatedly called.
! Therefore, the NEW_SOLID_MASS is actually just the solid formed in the
! current step.  Add this to previously solidified material to find the
! total composition of the Scheil-solidified solid.
      NEW_SOLID_MASS = MASS_IN_PHASE(SOLNO,INITIAL)
! The total system is 100kg, so the fractional mass that has transformed
! is 0.01 * the new mass
      SOLIDFRAC = SOLIDFRAC + 1.0D-02 * NEW_SOLID_MASS
! find rate of solid formation with decreasing temperature.  The total
! mass of the system ever used si 100kg, although some of that has
! solidified in the recalescence step, so is not used in thermodynamic
! calculations here, but must be accounted for when finding what
! fraction of the material solidifies in any one step.
      DFDT = (NEW_SOLID_MASS / 1.0D+02) / (STARTTEMP - ENDTEMP)
      DO I10=1,INIT_NO_OF_COMPONENTS(),1
! RESET THE LIQUID COMPOSITION in this subroutine to THE REMAINING
! LIQUID IN THE SYSTEM
        LIQ_COMP(I10) = COMPONENT_MASS_IN_PHASE(I10,LIQNO,INITIAL)
! ADD THE NEW SOLID TO THE EXISTING SOLID FORMED SCHEIL SOLIDIFICATION.
        SOLID_COMP(I10,K10) = SOLID_COMP(I10,K10) + 
     & COMPONENT_MASS_IN_PHASE(I10,SOLNO,INITIAL)
      END DO
!----------------------------------------------------------------------!
      END SUBROUTINE SCHEIL
!**********************************************************************!
! subroutine to find the peritectic temperature.  The subroutine allows
! liquid, austenite and ferrite in calcualtions and finds the
! temperature at which austenite first appears upo cooling.  It could
! also find the temperature for which liquid and ferrite has the same
! free energy as liquid and austenite
      SUBROUTINE FIND_PERITECTIC(SYSCOMP,PERI_TEMP,LIQUID,AUSTENITE,
     & FERRITE)
      IMPLICIT NONE
      INCLUDE 'DIRUSRAP.FOR'
!----------------------------------------------------------------------!
! declare variables.
! I12, J12 are local dummy variables.
      INTEGER I12,J12
! variables are passed from main program.
      DOUBLE PRECISION SYSCOMP,PERI_TEMP,PERI_START
! local variables (only exist in this subroutine).
      DOUBLE PRECISION TEMP_TRY,TOLERANCE,TRIAL_STEP
! phase names.
      CHARACTER*20 AUSTENITE,FERRITE,LIQUID,BLANKPHASE
! define size of arrays.
      DIMENSION SYSCOMP(12),TEMP_TRY(2)
!----------------------------------------------------------------------!
! set the maximum permissible mass fraction of austenite that can be
! used to define the peritectic.
      TOLERANCE  = 1.0D-04
! set the temperature steps for finding the approximate peritectic.
      TRIAL_STEP = 1.0D+01
!set a too-high initial guess of the peritectic so it can be refined.
      PERI_START = 1.8D+03
!----------------------------------------------------------------------!
! set upper bound to current temperature and lower bound to 600C
! (definitely below peritectic).
      TEMP_TRY(1) = PERI_START
      TEMP_TRY(2) = PERI_START
      DO I12=1,INIT_NO_OF_COMPONENTS()
        CALL SET_INIT_COMPONENT_MASS(I12,SYSCOMP(I12))
      END DO
! examine the mass of austenite formed at the test temperature, starting
! with the maximum recalescence temperature.
      PERI_REFINER:DO
! allow austenite, ferrite and liquid in the calculation.
        do i12=1,init_no_of_phases(i12)
          if(init_phase_name(i12) == liquid .or. 
     & init_phase_name(i12) == ferrite .or. 
     & init_phase_name(i12) == austenite) then
            call set_init_phase_class(i12,normal)
          else
            call set_init_phase_class(i12,absent)
          end if
        end do
! set the temperature of the calculation to the current temperature
        CALL SET_TEMPERATURE(TEMP_TRY(2)+2.7315D+02)
! calculate the equilibrium
        CALL COMPUTE_EQUILIBRIUM()
! save the Gibbs energy of the LIQUID + FERRITE mixture
        IF(MASS_IN_PHASE(INIT_PHASE_NO(AUSTENITE),INITIAL)
     & .GE. 1.0D-05) THEN
          IF(MASS_IN_PHASE(INIT_PHASE_NO(AUSTENITE),INITIAL)
     & / SUM_OF_COMPONENT_MASS() .LE. TOLERANCE .OR. 
     & TRIAL_STEP .LE. 1.0D-03) THEN
            EXIT PERI_REFINER
          ELSE
            TRIAL_STEP = 1.0D-01 * TRIAL_STEP
            TEMP_TRY(2) = TEMP_TRY(1) - TRIAL_STEP
          END IF
        ELSE
          TEMP_TRY(1) = TEMP_TRY(2)
          TEMP_TRY(2) = TEMP_TRY(2) - TRIAL_STEP
        END IF
      END DO PERI_REFINER
!----------------------------------------------------------------------!
      END SUBROUTINE FIND_PERITECTIC
!**********************************************************************!
      SUBROUTINE DO_PERITECTIC(THIS_TEMP,SOLIDFRAC,THE_TIME_STEP,
     & THE_TEMP_RATE,LIQ_COMP,SOLID_COMP,AUSTENITE,LIQUID,THE_TIME)
      IMPLICIT NONE
      INCLUDE 'DIRUSRAP.FOR'
!----------------------------------------------------------------------!
! I13 is a dummy variable set only locally in this subroutine
      INTEGER I13
! all the following variables are global and passed to the subroutine
      DOUBLE PRECISION THIS_TEMP,SOLIDFRAC,THE_TEMP_RATE,LIQ_COMP
      DOUBLE PRECISION SOLID_COMP,THE_TIME_STEP,LIQ_MASS,DSOLID,THE_TIME
! and the following variables are local only
      DOUBLE PRECISION LIQ_GIBBS,AUS_GIBBS,DELTA_G_PERI,DFRAC,MAX_PERI
! all character varibales (phase names for MTData) are passed to the
! subroutine and are global
      CHARACTER*20 LIQUID,AUSTENITE
! all composition sets are global and passed to the subroutine.  Other
! variable sare local.
      DIMENSION LIQ_COMP(12),SOLID_COMP(12,4)
      DIMENSION MAX_PERI(12)
!----------------------------------------------------------------------!
      CALL SET_TEMPERATURE(THIS_TEMP+2.7315D+02)
!----------------------------------------------------------------------!
      DO I13=1,INIT_NO_OF_PHASES(),1
        IF(INIT_PHASE_NAME(I13) == LIQUID .OR. 
     & INIT_PHASE_NAME(I13) == AUSTENITE) THEN
          CALL SET_INIT_PHASE_CLASS(I13,NORMAL)
        ELSE
          CALL SET_INIT_PHASE_CLASS(I13,ABSENT)
        END IF
      END DO
      CALL COMPUTE_EQUILIBRIUM()

      DO I13=1,INIT_NO_OF_COMPONENTS()
        MAX_PERI(I13) = 
     & COMPONENT_W_OF_PHASE(I13,INIT_PHASE_NO(LIQUID),INITIAL)
      END DO
!----------------------------------------------------------------------!
      PERI_SOLIDIFICATION:DO

        DO I13=1,INIT_NO_OF_COMPONENTS()
          CALL SET_INIT_COMPONENT_MASS(I13,LIQ_COMP(I13))
        END DO

        DO I13=1,INIT_NO_OF_PHASES(),1
          IF(INIT_PHASE_NAME(I13) == LIQUID) THEN
            CALL SET_INIT_PHASE_CLASS(I13,NORMAL)
          ELSE
            CALL SET_INIT_PHASE_CLASS(I13,ABSENT)
          END IF
        END DO
        CALL COMPUTE_EQUILIBRIUM()
        LIQ_GIBBS = SYSTEM_GIBBS_ENERGY / SUM_OF_COMPONENT_MASS()
!----------------------------------------------------------------------!
        DO I13=1,INIT_NO_OF_PHASES(),1
          IF(INIT_PHASE_NAME(I13) == LIQUID .OR. 
     & INIT_PHASE_NAME(I13) == AUSTENITE) THEN
            CALL SET_INIT_PHASE_CLASS(I13,NORMAL)
          ELSE
            CALL SET_INIT_PHASE_CLASS(I13,ABSENT)
          END IF
        END DO
        CALL COMPUTE_EQUILIBRIUM()
        AUS_GIBBS = SYSTEM_GIBBS_ENERGY / SUM_OF_COMPONENT_MASS()
!----------------------------------------------------------------------!
        DELTA_G_PERI = AUS_GIBBS - LIQ_GIBBS
!----------------------------------------------------------------------!
        DFRAC = THE_TIME_STEP * THE_TEMP_RATE / DELTA_G_PERI
        SOLIDFRAC = SOLIDFRAC + DFRAC
!----------------------------------------------------------------------!
        LIQ_MASS = 0.0D+00
        DO I13=1,INIT_NO_OF_COMPONENTS(),1
          SOLID_COMP(I13,3) = SOLID_COMP(I13,3) + DFRAC 
     & * COMPONENT_W_OF_PHASE(I13,INIT_PHASE_NO(AUSTENITE),INITIAL)
          LIQ_COMP(I13) = LIQ_COMP(I13) - DFRAC 
     & * COMPONENT_W_OF_PHASE(I13,INIT_PHASE_NO(AUSTENITE),INITIAL)
        LIQ_MASS = LIQ_MASS + LIQ_COMP(I13)
        END DO
!----------------------------------------------------------------------!
        DO I13=1,INIT_NO_OF_PHASES(),1
          IF(INIT_PHASE_NAME(I13) == LIQUID .OR. 
     & INIT_PHASE_NAME(I13) == AUSTENITE) THEN
            CALL SET_INIT_PHASE_CLASS(I13,NORMAL)
          ELSE
            CALL SET_INIT_PHASE_CLASS(I13,ABSENT)
          END IF
        END DO
        CALL COMPUTE_EQUILIBRIUM()
        IF(MASS_IN_PHASE(INIT_PHASE_NO(AUSTENITE),INITIAL) 
     & .LE. 1.0D-04) THEN
          EXIT PERI_SOLIDIFICATION
        END IF
        IF(SOLIDFRAC .GE. 9.990D-01) THEN
          EXIT PERI_SOLIDIFICATION
        END IF
!----------------------------------------------------------------------!
        THE_TIME = THE_TIME + THE_TIME_STEP
        PRINT 1001, THE_TIME,THIS_TEMP,SOLIDFRAC
      END DO PERI_SOLIDIFICATION
!----------------------------------------------------------------------!
 1001 FORMAT('peri ',E15.7,1X,F8.3,1X,E12.4)
!----------------------------------------------------------------------!
      END SUBROUTINE DO_PERITECTIC
!**********************************************************************!
!**********************************************************************!
!----------------------------------------------------------------------!
!-----------------------------MAIN PROGRAM-----------------------------!
!----------------------------------------------------------------------!
!**********************************************************************!
!**********************************************************************!
      PROGRAM DROPPER
      IMPLICIT NONE
      INCLUDE 'DIRUSRAP.FOR'
!**********************************************************************!
! declare variables
      INTEGER I,J,IOCHECK,MTMODE,NO_PHASES,PERI_POSS,LIQNO,CNO
      INTEGER PERI_LIMIT,COMPLETENESS
!----------------------------------------------------------------------!
      REAL DIAMETER,SOLID_FRAC,DROP_SPEC_HEAT,SOL_SPEC_HEAT
      REAL LIQ_SPEC_HEAT,DROP_DENSE,HEAT_TRANS_COEFF,TIME_STEP
      REAL EMISSIVITY,GAS_TEMP,WALL_TEMP,TEMP_RATE,RADI_RATE
      REAL CONVEC_RATE,THE_TIME
      DOUBLE PRECISION LIQUIDUS,LIQFER,LIQAUS,DELTA_T_HOM,DELTA_T_HET
      DOUBLE PRECISION THE_TEMP,TOT_COMP,SOL_COMP,LIQ_COMP,LATENT,DELTAH
      DOUBLE PRECISION NUMBER_NUCLEI,NUC_NUM,NUCRATE,NUC_TEMP,VOLUME
      DOUBLE PRECISION STARTING_TEMP,AUS_SPEC_HEAT,FER_SPEC_HEAT
      DOUBLE PRECISION FER_HEAT_CAP,AUS_HEAT_CAP,SOLID_HEAT_CAP,PPTSIZE
      DOUBLE PRECISION SOLID_DENSE,EVOLVE_RATE,SOLIDFRAC,MUSHY_SPEC_HEAT
      DOUBLE PRECISION RECAL_TEMP_RATE,RECAL_T_STEP,TM_FE,SOLID_COMP
      DOUBLE PRECISION DFDT,DELTA_G,STARTTEMP,ENDTEMP,SOL_DENSE
      DOUBLE PRECISION LIQ_DENSE,MUSHY_DENSE,PERI_TEMP,END_TEMP
      DIMENSION TOT_COMP(12),SOL_COMP(12),LIQ_COMP(12),LATENT(3)
      DIMENSION DELTAH(2),NUC_NUM(2),SOLID_COMP(12,4)
      DOUBLE PRECISION PI,AVOGADRO,BOLTZMANN,STEF_BOLT
      PARAMETER(PI=3.14159265359D+00,AVOGADRO=6.02214086D+23)
      PARAMETER(BOLTZMANN=1.38064852D-23,STEF_BOLT=5.6703D-08)
      PARAMETER(TM_FE=1.53779478D+03)
!----------------------------------------------------------------------!
      CHARACTER*(*) INFILE,AUSTENITE,FERRITE,LIQUID,BLANKPHASE
      CHARACTER*20 SOLIDPHASE
      PARAMETER(INFILE='input1.txt')
      PARAMETER(AUSTENITE='FCC_A1              ')
      PARAMETER(FERRITE='BCC_A2                ')
      PARAMETER(LIQUID='LIQUID                ')
      PARAMETER(BLANKPHASE = 'abcdefghijklmnopqrst')
      CHARACTER*20 MPIFILE,MPRFILE
!**********************************************************************!
! initialise variables to implausible values to avoid risk they are
! incorrectly used before being assigned and the mistake is not found
! or to realistic values so they may be used.

      COMPLETENESS     = 0
      TEMP_RATE        = -3.9D+12
      DIAMETER         = -100.0D-06
      THE_TEMP         = -1.80D+03
      GAS_TEMP         = -1.0D+02
      WALL_TEMP        = -5.0D+01
      DROP_SPEC_HEAT   = -8.0D+02
      DROP_DENSE       = -6.23D-05
      SOL_DENSE        = -3831.141
      LIQ_DENSE        = -1.33d+21
      MUSHY_DENSE      = 3.67D+19
      TIME_STEP        = -1.0D-03
      HEAT_TRANS_COEFF = -7.0D+00
      EMISSIVITY       = -2.0D-01
      RADI_RATE        = -6.9D+00
      CONVEC_RATE      = -1.78D+05
      RECAL_TEMP_RATE  = 21312.434D-02
      THE_TIME         = 0.0D+00
      PERI_TEMP        = 1.5243D+22
      LIQ_SPEC_HEAT    = -5.6D+12
      SOL_SPEC_HEAT    = -3.4D+01
      FER_HEAT_CAP     = -7.6D+02
      AUS_HEAT_CAP     = -8.2D+00
      SOLID_HEAT_CAP   = 2234234.5345
      MUSHY_SPEC_HEAT  = 45966.133123
      SOLID_FRAC       = 0.0D+00
      NUCRATE          = -4.823D+23
      NUC_NUM(1)       = 0.0D+00
      NUC_NUM(2)       = 0.0D+00
      NUC_TEMP         = 0.0D+00
      RECAL_T_STEP     = 1.0D-07
      VOLUME           = -1.633D+12
      END_TEMP         = 343.313D+01
      MTMODE           = 0
      PERI_POSS        = 0
      PERI_LIMIT       = 0
      LIQNO            = 0
      MPIFILE          = '                    '
      MPRFILE          = '                    '
      INIT_COMP:DO I=1,12,1
        TOT_COMP(I)    = 0.0D+00
        LIQ_COMP(I)    = 0.0D+00
        SOL_COMP(I)    = 0.0D+00
        DO J=1,3,1
          SOLID_COMP(I,J) = 0.0D+00
        END DO
      END DO INIT_COMP
      INIT_GIBBS:DO I=1,3,1
        LATENT(I)       = 0.0D+00
      END DO INIT_GIBBS

      OPEN(UNIT=13,FILE=INFILE,STATUS='OLD')
        READ(UNIT=13,FMT=*,IOSTAT=IOCHECK) DIAMETER
        READ(UNIT=13,FMT=*,IOSTAT=IOCHECK) STARTING_TEMP
        READ(UNIT=13,FMT=*,IOSTAT=IOCHECK) END_TEMP
        READ(UNIT=13,FMT=*,IOSTAT=IOCHECK) GAS_TEMP
        READ(UNIT=13,FMT=*,IOSTAT=IOCHECK) WALL_TEMP
        READ(UNIT=13,FMT=*,IOSTAT=IOCHECK) DROP_SPEC_HEAT
        READ(UNIT=13,FMT=*,IOSTAT=IOCHECK) LIQ_SPEC_HEAT
        READ(UNIT=13,FMT=*,IOSTAT=IOCHECK) SOL_SPEC_HEAT
        READ(UNIT=13,FMT=*,IOSTAT=IOCHECK) TIME_STEP
        READ(UNIT=13,FMT=*,IOSTAT=IOCHECK) HEAT_TRANS_COEFF
        READ(UNIT=13,FMT=*,IOSTAT=IOCHECK) EMISSIVITY
        COMPREAD:DO I=2,12,1
          READ(UNIT=13,FMT=*,IOSTAT=IOCHECK) TOT_COMP(I)
        END DO COMPREAD
        READ(UNIT=13,FMT=*,IOSTAT=IOCHECK) MPIFILE
        READ(UNIT=13,FMT=*,IOSTAT=IOCHECK) MPRFILE
      CLOSE(UNIT=13,STATUS='KEEP')
!----------------------------------------------------------------------!
! start MTData to be available for calculataions
      CALL START_MTDATA(MTMODE,MPIFILE,MPRFILE)
! set the temperature and then the pressure to atmospheric pressure.
      CALL SET_TEMPERATURE(THE_TEMP+2.7315E+02)
      CALL SET_PRESSURE(1.013250D+05)
! FIND THE NUMBER OF POSSIBLE PHASES AND SET THIS TO THE VALUE OF
! NO_PHASES.
      NO_PHASES=INIT_NO_OF_PHASES()
      print *, 'initial conditions set.'
!----------------------------------------------------------------------!
! find the liquidus for use in nucleation claculations.
      CALL FIND_LIQ(TOT_COMP,LIQFER,LIQAUS,LIQUID,AUSTENITE,FERRITE,
     & NO_PHASES)
      print *, 'liquidus tempeatures found',LIQFER,LIQAUS
!----------------------------------------------------------------------!
! simulate the cooling from superheated melt to find the nucleation
! temperature of solid.
      THE_TEMP = STARTING_TEMP
      print *, 'finding nucleation temperature'
      NUCFIND:DO
      print *, THE_TEMP
! calculate the density of the molten droplet, based on carbon content
! and temperature
      DROP_DENSE = 1.0E+03 * (7.10E+00-7.32E-02*TOT_COMP(2)) -
     & (THE_TEMP-1.823E+03) * 1.0E-01 * (8.28e+00-8.74e-01*TOT_COMP(2))
! THE SYSTEM IS DEFINED AS CONSISTING OF 100kg.  THIS ALLOWS THE USER
! TO INPUT wt% EASILY.  Use this to find the volume from the density.
      VOLUME = 1.0D+02 / DROP_DENSE
!----------------------------------------------------------------------!
            CALL DROP_COOLING_RATE(TEMP_RATE,DIAMETER,THE_TEMP,
     & GAS_TEMP,WALL_TEMP,DROP_SPEC_HEAT,DROP_DENSE,
     & EMISSIVITY,STEF_BOLT,HEAT_TRANS_COEFF,TOT_COMP,
     & RADI_RATE,CONVEC_RATE)

! COMPUTE AN EQUILIBRIUM FOR LIQUID AT THE_TEMP FOR THE BULK COMPOSITION
            CALL DRIVE(THE_TEMP,TOT_COMP,LIQUID,NO_PHASES,BLANKPHASE)
! FOR NOW, DO NOT CORRECT FOR MASS OR TOTAL NUMBER OF MOLES
            LATENT(1) = SYSTEM_GIBBS_ENERGY()
! COMPUTE AN EQUILIBRIUM FOR FERRITE AT THE_TEMP FOR THE BULK
! COMPOSITION
            CALL DRIVE(THE_TEMP,TOT_COMP,FERRITE,NO_PHASES,BLANKPHASE)
! FOR NOW, DO NOT CORRECT FOR MASS OR TOTAL NUMBER OF MOLES
            LATENT(2) = SYSTEM_GIBBS_ENERGY()
! COMPUTE AN EQUILIBRIUM FOR AUSTENITE AT THE_TEMP FOR THE BULK
! COMPOSITION
            CALL DRIVE(THE_TEMP,TOT_COMP,AUSTENITE,NO_PHASES,BLANKPHASE)
! FOR NOW, DO NOT CORRECT FOR MASS OR TOTAL NUMBER OF MOLES
            LATENT(3) = SYSTEM_GIBBS_ENERGY()
! ALL THREE ENTHALPIES ARE CALCULATED FOR THE SAME SYSTEM, SO ARE ALWAYS
! FOR THE SAME AMOUNT OF MATERIAL
            DELTAH(1) = LATENT(2)- LATENT(1)
! ALL THREE ENTHALPIES ARE CALCULATED FOR THE SAME SYSTEM, SO ARE ALWAYS
! FOR THE SAME AMOUNT OF MATERIAL
            DELTAH(2) = LATENT(3) - LATENT(1)

! if it is possible to form ferrite form the melt, calculate the number
! of nuclei per droplet.
            IF(DELTAH(1) .LT. 0.0D+00) THEN
              CALL NUCIT(THE_TEMP,TEMP_RATE,LIQFER,DIAMETER,PI,AVOGADRO,
     & BOLTZMANN,DELTAH(1),VOLUME,NUMBER_NUCLEI)
! if the number of nuclei per droplet is 1 or more, assume solid is
! fomring and move on to the next phase of the program
              NUC_NUM(1) = NUMBER_NUCLEI
              IF(NUC_NUM(1) .GE. 1.0D+00) THEN
                DELTA_T_HOM = LIQFER -  THE_TEMP
                DELTA_T_HET = DELTA_T_HOM * 
! CALCULATIONS ASSUME HOMOGENEOUS NUCLEATION.  CORRECT FOR HETEROGENEOUS
! NUCLEATION.
     & EXP(-2.2D+12*PI*DIAMETER**3.0D+00 / 6.0D+00)
                NUC_TEMP = LIQFER - DELTA_T_HET
! ALLOW THE PERITECTIC TO BE CONSIDERED LATER IN THE PROGRAM
                PERI_POSS = 1
                EXIT NUCFIND
              END IF
            END IF

            IF(DELTAH(2) .LT. 0.0D+00) THEN
              CALL NUCIT(THE_TEMP,TEMP_RATE,LIQAUS,DIAMETER,PI,AVOGADRO,
     & BOLTZMANN,DELTAH(2),VOLUME,NUMBER_NUCLEI)
              NUC_NUM(2) = NUMBER_NUCLEI
! if the number of nuclei per droplet is 1 or more, assume solid is
! forming and move on to the next phase of the program.
              IF(NUC_NUM(2) .GE. 1.0D+00) THEN
                DELTA_T_HOM = LIQAUS -  THE_TEMP
                DELTA_T_HET = DELTA_T_HOM * 
! CALCULATIONS ASSUME HOMOGENEOUS NUCLEATION.  CORRECT FOR HETEROGENEOUS
! NUCLEATION.
     & EXP(-2.2D+12*PI*DIAMETER**3.0D+00 / 6.0D+00)
                NUC_TEMP = LIQAUS - DELTA_T_HET
! DO NOT ALLOW THE PERITECTIC TO BE CONSIDERED LATER IN THE PROGRAM
! (ALREADY FORMS AUSTENITE, SO NO NEW PHASE TO CONSIDER AS LIQUID
! BECOMES MORE SOLUTE-RICH).
                PERI_POSS = 0
                EXIT NUCFIND
              END IF
            END IF
 
            THE_TEMP = THE_TEMP + TIME_STEP * TEMP_RATE
            THE_TIME = THE_TIME + TIME_STEP

      END DO NUCFIND
!**********************************************************************!
      THE_TEMP = STARTING_TEMP
      THE_TIME = 0.0D+00
! allow the liquid to cool to the nucleation tempeature and record the
! temperature and time.
      PRINT 100
      COOL:DO

        CALL DROP_COOLING_RATE(TEMP_RATE,DIAMETER,THE_TEMP,
     & GAS_TEMP,WALL_TEMP,DROP_SPEC_HEAT,DROP_DENSE,
     & EMISSIVITY,STEF_BOLT,HEAT_TRANS_COEFF,TOT_COMP,
     & RADI_RATE,CONVEC_RATE)

        THE_TEMP = THE_TEMP + TIME_STEP * TEMP_RATE
        THE_TIME = THE_TIME + TIME_STEP

        PRINT 101, THE_TIME,THE_TEMP

        IF(THE_TEMP .LE. NUC_TEMP) THEN
          EXIT COOL
        END IF

      END DO COOL
!**********************************************************************!
! set the recalescence up to use austenite or ferrite as calculated
! earlier
      IF(PERI_POSS == 1) THEN
        SOLIDPHASE = FERRITE
        LIQUIDUS = LIQFER
        SOLID_HEAT_CAP = 23.94 + 9.6D-3 * (THE_TEMP+2.7315D+02)
      ELSE IF(PERI_POSS == 0) THEN
        SOLIDPHASE = AUSTENITE
        LIQUIDUS = LIQAUS
        SOLID_HEAT_CAP = 20.00 + 6.6D-03 * (THE_TEMP+2.7315D+02)
      ELSE
        PRINT *, 'Error in initial solidification, program exits.'
        STOP
      END IF
! set the solid fraction and precipitate size to zero - no significant
! solid fraction at point of nucleation.
      SOLID_FRAC = 0.0D+00
      PPTSIZE = 0.0D+00
      PPTSIZE = PPTSIZE + 0.5D+00 * RECAL_T_STEP * 1.0D-2 * 
     & (LIQUIDUS - THE_TEMP)
! set the liquid composition equal to the bulk as at the point of
! nucleation, the entire sample is molten.
      DO I=2,INIT_NO_OF_COMPONENTS()
        LIQ_COMP(I) = TOT_COMP(I)
      END DO
      CALL SETCOMP(LIQ_COMP)
      LIQ_COMP(1) = INIT_COMPONENT_MASS(1)
! allow primary solid to form and account for the latent heat of
! transformation when determining the tempeature change.
      RECAL:DO
! find the density of liquid and solid phases at the current temperature
      DROP_DENSE = 1.0E+03 * (7.10E+00-7.32E-02*LIQ_COMP(2)) -
     & (THE_TEMP-1.823E+03) * 1.0E-01 * (8.28e+00-8.74e-01*LIQ_COMP(2))
      IF(SOLIDPHASE == AUSTENITE) THEN
! http://www.bssa.org.uk/topics.php?article=139
        SOLID_DENSE = 7.970D+03 - 0.42 * THE_TEMP
      ELSE IF(SOLIDPHASE == FERRITE) THEN
! Christien et al., Mater. Char. 82(2013)50–57
        SOLID_DENSE = 7.95D+03 - 0.46 * THE_TEMP
      ELSE
        PRINT *, 'ERROR IN FINDING SOLID/LIQUID DENSITY. PROGRAM EXITS.'
        STOP
      END IF
      MUSHY_DENSE = SOLIDFRAC * SOLID_DENSE + (1.0D+00-SOLIDFRAC) * 
     & DROP_DENSE
! find the change in solid fraction in one timestep and the rate of heat
! evolution
      CALL RECALESCENCE(EVOLVE_RATE,THE_TEMP,SOLIDFRAC,PPTSIZE,
     & DBLE(DIAMETER),LIQUIDUS,LIQ_COMP,LIQUID,SOLIDPHASE,
     & DBLE(MUSHY_DENSE),PI,RECAL_T_STEP)
! find the heat capacity of the mushy droplet (weighted average fo the
! solid and liquid heat capacities)
      MUSHY_SPEC_HEAT = SOLIDFRAC*SOLID_HEAT_CAP + 
     & (1.0D+00-SOLIDFRAC)*LIQ_SPEC_HEAT
! find the temperature change
      RECAL_TEMP_RATE = EVOLVE_RATE / MUSHY_SPEC_HEAT
! find the cooling rate due to radiation and convection
      CALL DROP_COOLING_RATE(TEMP_RATE,DIAMETER,THE_TEMP,
     & GAS_TEMP,WALL_TEMP,DROP_SPEC_HEAT,DROP_DENSE,
     & EMISSIVITY,STEF_BOLT,HEAT_TRANS_COEFF,TOT_COMP,
     & RADI_RATE,CONVEC_RATE)
      TEMP_RATE = TEMP_RATE + RECAL_TEMP_RATE
      PRINT 102, THE_TIME,THE_TEMP,SOLIDFRAC,SOLIDPHASE
! stop this calculation if the temperature change rate is not positive.
! This defines the end of recalescence.
      IF(TEMP_RATE .LE. 0.0D+00 .AND. PPTSIZE .GT. 0.0D+00) THEN
        EXIT RECAL
      ELSE IF(SOLIDFRAC .GE. 9.990D-01) THEN
        EXIT RECAL
      ELSE
! add the contributions of temperature change due to recalescence and
! radiative and convective cooling
        THE_TEMP = THE_TEMP + RECAL_T_STEP * TEMP_RATE
        THE_TIME = THE_TIME + RECAL_T_STEP
      END IF
! SET THE MASS OF IRON IN THE TOTAL COMPOSITION ARRAY.
      TOT_COMP(1) = INIT_COMPONENT_MASS(1)
      DO I=1,INIT_NO_OF_COMPONENTS(),1
! RECORD THE MASS OF THE COMPONENTS IN THE INITIAL SOLID, I.E. NO
! PARTITIONING.
        SOLID_COMP(I,1) = TOT_COMP(I) * SOLIDFRAC
! SET THE MASS OF COMPONENTS IN THE LIQUID
        LIQ_COMP(I)     = TOT_COMP(I) * (1.0D+00 - SOLIDFRAC)
      END DO
!----------------------------------------------------------------------!
      END DO RECAL
!**********************************************************************!
! allow Scheil solidification until the peritectic reaction (if any)
! begins.  Assume peritectic reaction occurs if LIQ --> LIQ + FER goes
! from higher to smaller driivng force than LIQ --> LIQ + AUS
!----------------------------------------------------------------------!
! first find the heat lost in the time step per kg of material due to
! convection and radiation.
      CALL DROP_COOLING_RATE(TEMP_RATE,DIAMETER,THE_TEMP,
     & GAS_TEMP,WALL_TEMP,DROP_SPEC_HEAT,DROP_DENSE,
     & EMISSIVITY,STEF_BOLT,HEAT_TRANS_COEFF,TOT_COMP,
     & RADI_RATE,CONVEC_RATE)
! guess the end temperature based on the radiative/convective cooling
! rate - this will be an oversetimate as no latent heat isa accounted
! for, but is as good a starting point as any.  This is needed to assess
! rate of change of solid fractino with temperature, df/dT.
      STARTTEMP = DBLE(THE_TEMP)
      ENDTEMP   = STARTTEMP + TEMP_RATE * TIME_STEP
!----------------------------------------------------------------------!
! find the peritectic temperature
      CALL FIND_PERITECTIC(LIQ_COMP,PERI_TEMP,LIQUID,AUSTENITE,FERRITE)
! allow system to form a solid+liquid equilirbium so the behaviour may
! be determined.  The "2" is to fill the second column of the solid
! composition array.
      CALL SCHEIL(LIQ_COMP,STARTTEMP,ENDTEMP,LIQUID,SOLIDPHASE,
     & BLANKPHASE,SOLID_COMP,DELTA_G,DBLE(DFDT),DBLE(TIME_STEP),LIQNO,
     & SOLIDFRAC,2)
!----------------------------------------------------------------------!
! test to see if the solid is stable wrt. s+l and if so, throw the
! simulation back to the recalescence subroutint to model the
! solidification.
      IF(MASS_IN_PHASE(LIQNO,INITIAL) <= 1.0D-4) THEN
        IF(THE_TEMP .LE. PERI_TEMP) THEN
          SOLIDPHASE = AUSTENITE
          PERI_POSS  = 0
        END IF
        RECAL2:DO
          DROP_DENSE = 1.0E+03 * (7.10E+00-7.32E-02*LIQ_COMP(2)) -
     & (THE_TEMP-1.823E+03) * 1.0E-01 * (8.28e+00-8.74e-01*LIQ_COMP(2))
          IF(SOLIDPHASE == AUSTENITE) THEN
! http://www.bssa.org.uk/topics.php?article=139
            SOLID_DENSE = 7.970D+03 - 0.42 * THE_TEMP
          ELSE IF(SOLIDPHASE == FERRITE) THEN
! Christien et al., Mater. Char. 82(2013)50–57
            SOLID_DENSE = 7.95D+03 - 0.46 * THE_TEMP
          ELSE
            PRINT *, 'ERROR IN FINDING SOLID/LIQUID DENSITY. PROGRAM ',
     & 'EXITS.'
            STOP
          END IF
          MUSHY_DENSE = SOLIDFRAC * SOLID_DENSE + (1.0D+00-SOLIDFRAC) * 
     & DROP_DENSE
          CALL RECALESCENCE(EVOLVE_RATE,THE_TEMP,SOLIDFRAC,PPTSIZE,
     & DBLE(DIAMETER),LIQUIDUS,LIQ_COMP,LIQUID,SOLIDPHASE,
     & DBLE(MUSHY_DENSE),PI,RECAL_T_STEP)
! find the heat capacity of the mushy droplet (weighted average of the
! solid and liquid heat capacities)
          MUSHY_SPEC_HEAT = SOLIDFRAC*SOLID_HEAT_CAP + 
     & (1.0D+00-SOLIDFRAC)*LIQ_SPEC_HEAT
! find the temperature change
          RECAL_TEMP_RATE = EVOLVE_RATE / MUSHY_SPEC_HEAT
! find the cooling rate due to radiation and convection
          CALL DROP_COOLING_RATE(TEMP_RATE,DIAMETER,THE_TEMP,
     & GAS_TEMP,WALL_TEMP,DROP_SPEC_HEAT,DROP_DENSE,
     & EMISSIVITY,STEF_BOLT,HEAT_TRANS_COEFF,TOT_COMP,
     & RADI_RATE,CONVEC_RATE)
          TEMP_RATE = TEMP_RATE + RECAL_TEMP_RATE
          THE_TIME = THE_TIME + RECAL_T_STEP
          PRINT 102, THE_TIME,THE_TEMP,SOLIDFRAC,SOLIDPHASE
          IF(SOLIDFRAC >= 9.990D-1) THEN
            SOLIDFRAC = 1.0D+00
            PRINT 102, THE_TIME,THE_TEMP,SOLIDFRAC,SOLIDPHASE
            COMPLETENESS = 1
            EXIT RECAL2
          END IF
        END DO RECAL2
      ELSE
!----------------------------------------------------------------------!
! if the remaining liquid in the system can form a solid/liquid
!equilibrium mixture, allow Scheil solidification to occur until
!either the peritectic or solidification is complete, whichever is
! first.
        IF(THE_TEMP .LE. PERI_TEMP) THEN
          SOLIDPHASE = AUSTENITE
          PERI_POSS  = 0
        END IF
          SCHEIL2:DO
!----------------------------------------------------------------------!
            IF(SOLIDFRAC >= 9.990D-01) THEN
! allow the simulation to end at 99.9% solid to avoid long 
!caulculations to transform virtually no liquid.  Stop the
!transformation if drop is all solid.  This is different to the similar
! IF statement at the end of the DO loop since it doesn't print any
! message.  These loops could be consolidated in future iterations of
! the program.
              SOLIDFRAC  = 1.0D+00
              EXIT SCHEIL2
            END IF
!----------------------------------------------------------------------!
! find the temperature-dependent and composition-dependent partition
! coefficients for each element
!      CALL PARTITION(LIQ_COMP,LIQUID,SOLIDPHASE,BLANKPHASE,THE_TEMP,K,
!     & KAVERAGE)
!----------------------------------------------------------------------!
! the "4" allows new mass in solid to be put into the fourth column of
! the solid composition array.
            CALL SCHEIL(LIQ_COMP,STARTTEMP,ENDTEMP,LIQUID,SOLIDPHASE,
     & BLANKPHASE,SOLID_COMP,DELTA_G,DBLE(DFDT),DBLE(TIME_STEP),LIQNO,
     & SOLIDFRAC,4)
! allow for the case where the remaining liquid undergoes bulk
! transformation to solid after recalescence.
!----------------------------------------------------------------------!
! find the average density of the droplet.
        DROP_DENSE = 1.0E+03 * (7.10E+00-7.32E-02*LIQ_COMP(2)) -
     & (THE_TEMP-1.823E+03) * 1.0E-01 * (8.28e+00-8.74e-01*LIQ_COMP(2))
        IF(SOLIDPHASE == AUSTENITE) THEN
! http://www.bssa.org.uk/topics.php?article=139
          SOLID_DENSE = 7.970D+03 - 0.42 * THE_TEMP
        ELSE IF(SOLIDPHASE == FERRITE) THEN
! Christien et al., Mater. Char. 82(2013)50–57
          SOLID_DENSE = 7.95D+03 - 0.46 * THE_TEMP
        ELSE
          PRINT *, 'ERROR IN FINDING SOLID/LIQUID DENSITY.  ',
     & 'PROGRAM EXITS.'
          STOP
        END IF
        MUSHY_DENSE = SOLIDFRAC * SOLID_DENSE + (1.0D+00-SOLIDFRAC) * 
     & DROP_DENSE
!----------------------------------------------------------------------!
! find the heat lost in the time step per kg of material due to
! convection and radiation.
            CALL DROP_COOLING_RATE(TEMP_RATE,DIAMETER,THE_TEMP,
     & GAS_TEMP,WALL_TEMP,DROP_SPEC_HEAT,SNGL(MUSHY_DENSE),EMISSIVITY,
     & STEF_BOLT,HEAT_TRANS_COEFF,TOT_COMP,RADI_RATE,CONVEC_RATE)
!----------------------------------------------------------------------!
! find the specific heat opf the mushy droplet
            MUSHY_SPEC_HEAT = SOLIDFRAC*SOLID_HEAT_CAP + 
     & (1.0D+00-SOLIDFRAC)*LIQ_SPEC_HEAT
! find the rate of temperature change, based on the heat loss and heat
! evolution by the transformation.
            TEMP_RATE = (RADI_RATE + CONVEC_RATE) / 
     & (MUSHY_SPEC_HEAT + DBLE(DFDT) * DELTA_G)
! step the temperature, based on the calculated time step and the
! temperature.
            THE_TIME = THE_TIME + TIME_STEP
            THE_TEMP = THE_TEMP + TIME_STEP * TEMP_RATE
!----------------------------------------------------------------------!
! set the start and end temperatures for the next loop
            STARTTEMP = THE_TEMP
            ENDTEMP   = THE_TEMP + TIME_STEP * TEMP_RATE
!----------------------------------------------------------------------!
! print out the results
            PRINT 102, THE_TIME,THE_TEMP,SOLIDFRAC,SOLIDPHASE
!----------------------------------------------------------------------!
            IF(SOLIDFRAC .GE. 9.990E-01) THEN
              SOLIDFRAC = 1.0D+02
              PRINT *, "SECOND SCHEIL PROCESS COMPLETES SOLIDIFICATION."
              EXIT SCHEIL2
            ! why have I done this - exit the Scheil lopo if peritectic
            ! can form and the temperature is low enough?  I don't think
            ! this is posible due to the previous subroutines
            ELSE IF(PERI_POSS == 1 .AND. THE_TEMP .LE. PERI_TEMP) THEN
              EXIT SCHEIL2
            END IF
!----------------------------------------------------------------------!
        END DO SCHEIL2
!----------------------------------------------------------------------!
      END IF
!**********************************************************************!
      IF(PERI_POSS == 0) THEN
        SOLIDCOOL1:DO
!----------------------------------------------------------------------!
          IF(SOLIDPHASE == AUSTENITE) THEN
! http://www.bssa.org.uk/topics.php?article=139
            SOLID_DENSE = 7.970D+03 - 0.42 * THE_TEMP
          ELSE IF(SOLIDPHASE == FERRITE) THEN
! Christien et al., Mater. Char. 82(2013)50–57
            SOLID_DENSE = 7.95D+03 - 0.46 * THE_TEMP
          ELSE
            PRINT *, 'ERROR IN FINDING SOLID/LIQUID DENSITY. ',
     & 'PROGRAM EXITS.'
            STOP
          END IF
!----------------------------------------------------------------------!
          CALL DROP_COOLING_RATE(TEMP_RATE,DIAMETER,THE_TEMP,
     & GAS_TEMP,WALL_TEMP,DROP_SPEC_HEAT,SNGL(SOLID_DENSE),EMISSIVITY,
     & STEF_BOLT,HEAT_TRANS_COEFF,TOT_COMP,RADI_RATE,CONVEC_RATE)
!----------------------------------------------------------------------!
          THE_TIME = THE_TIME + TIME_STEP
          THE_TEMP = THE_TEMP + TIME_STEP * TEMP_RATE
          PRINT 101, THE_TIME, THE_TEMP
          IF(THE_TEMP .LE. END_TEMP) THEN
            PRINT *, 'COOLING COMPLETE.  PROGRAM ENDS SUCCESSFULLY.'
            STOP
          END IF
        END DO SOLIDCOOL1
!----------------------------------------------------------------------!
      END IF
!**********************************************************************!
       CALL DO_PERITECTIC(DBLE(THE_TEMP),SOLIDFRAC,DBLE(TIME_STEP),
     & DBLE(TEMP_RATE),LIQ_COMP,SOLID_COMP,AUSTENITE,LIQUID,
     & DBLE(THE_TIME))
!**********************************************************************!
      SOLIDCOOL2:DO
!----------------------------------------------------------------------!
        IF(SOLIDPHASE == AUSTENITE) THEN
! http://www.bssa.org.uk/topics.php?article=139
          SOLID_DENSE = 7.970D+03 - 0.42 * THE_TEMP
        ELSE IF(SOLIDPHASE == FERRITE) THEN
! Christien et al., Mater. Char. 82(2013)50–57
          SOLID_DENSE = 7.95D+03 - 0.46 * THE_TEMP
        ELSE
          PRINT *, 'ERROR IN FINDING SOLID/LIQUID DENSITY. ',
     & 'PROGRAM EXITS.'
          STOP
        END IF
!----------------------------------------------------------------------!
        CALL DROP_COOLING_RATE(TEMP_RATE,DIAMETER,THE_TEMP,
     & GAS_TEMP,WALL_TEMP,DROP_SPEC_HEAT,SNGL(SOLID_DENSE),EMISSIVITY,
     & STEF_BOLT,HEAT_TRANS_COEFF,TOT_COMP,RADI_RATE,CONVEC_RATE)
!----------------------------------------------------------------------!
        THE_TIME = THE_TIME + TIME_STEP
        THE_TEMP = THE_TEMP + TIME_STEP * TEMP_RATE
        PRINT 101, THE_TIME, THE_TEMP
        IF(THE_TEMP .LE. END_TEMP) THEN
          EXIT SOLIDCOOL2
        END IF
      END DO SOLIDCOOL2
!**********************************************************************!
      PRINT 103
!**********************************************************************!
 100  FORMAT('Time / s,Temperature / C,Atomic fraction of solid,'
     & 'Last solid phase to form')
 101  FORMAT(E15.7,',',F8.3)
 102  FORMAT(E15.7,',',F8.3,',',E12.4,',',A10)
 103  FORMAT("COOLING COMPLETE.  PROGRAM ENDS SUCCESSFULLY.")
!**********************************************************************!
      END PROGRAM DROPPER
